@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -355,12 +355,13 @@ int main(int argc, char **argv)
                   (char*) &opt_enable_cleartext_plugin);
   if (!opt_only_print) 
   {
-    if (!(mysql_real_connect(&mysql, host, user, opt_password,
-                             NULL, opt_mysql_port,
-                             opt_mysql_unix_port, connect_flags)))
+    if (!(mysql_connect_ssl_check(&mysql, host, user, opt_password,
+                                  NULL, opt_mysql_port, opt_mysql_unix_port,
+                                  connect_flags, opt_ssl_required)))
     {
       fprintf(stderr,"%s: Error when connecting to server: %s\n",
               my_progname,mysql_error(&mysql));
+      mysql_close(&mysql);
       free_defaults(defaults_argv);
       my_end(0);
       exit(1);
@@ -406,8 +407,7 @@ int main(int argc, char **argv)
   pthread_mutex_destroy(&sleeper_mutex);
   pthread_cond_destroy(&sleep_threshhold);
 
-  if (!opt_only_print) 
-    mysql_close(&mysql); /* Close & free connection */
+  mysql_close(&mysql); /* Close & free connection */
 
   /* now free all the strings we created */
   my_free(opt_password);
@@ -422,6 +422,7 @@ int main(int argc, char **argv)
 #ifdef HAVE_SMEM
   my_free(shared_memory_base_name);
 #endif
+  mysql_server_end();
   free_defaults(defaults_argv);
   my_end(my_end_arg);
 
@@ -1247,7 +1248,7 @@ get_options(int *argc,char ***argv)
       fprintf(stderr, "Invalid value specified for the option "
               "'number-int-cols'\n");
       option_cleanup(str);
-      return 1;
+      DBUG_RETURN(1);
     }
     num_int_cols= atoi(str->string);
     if (str->option)
@@ -1263,7 +1264,7 @@ get_options(int *argc,char ***argv)
       fprintf(stderr, "Invalid value specified for the option "
               "'number-char-cols'\n");
       option_cleanup(str);
-      return 1;
+      DBUG_RETURN(1);
     }
     num_char_cols= atoi(str->string);
     if (str->option)
@@ -1504,7 +1505,7 @@ get_options(int *argc,char ***argv)
     if(parse_option(default_engine, &engine_options, ',') == -1)
     {
       fprintf(stderr, "Invalid value specified for the option 'engine'\n");
-      return 1;
+      DBUG_RETURN(1);
     }
   }
 
@@ -1845,6 +1846,7 @@ pthread_handler_t run_task(void *p)
   {
     fprintf(stderr,"%s: mysql_thread_init() failed ERROR : %s\n",
             my_progname, mysql_error(mysql));
+    mysql_close(mysql);
     exit(0);
   }
 
@@ -1918,6 +1920,7 @@ limit_not_met:
           {
             fprintf(stderr,"%s: Cannot run query %.*s ERROR : %s\n",
                     my_progname, (uint)length, buffer, mysql_error(mysql));
+            mysql_close(mysql);
             exit(0);
           }
         }
@@ -1928,6 +1931,7 @@ limit_not_met:
         {
           fprintf(stderr,"%s: Cannot run query %.*s ERROR : %s\n",
                   my_progname, (uint)ptr->length, ptr->string, mysql_error(mysql));
+          mysql_close(mysql);
           exit(0);
         }
       }
@@ -1966,8 +1970,7 @@ end:
   if (commit_rate)
     run_query(mysql, "COMMIT", strlen("COMMIT"));
 
-  if (!opt_only_print) 
-    mysql_close(mysql);
+  mysql_close(mysql);
 
   mysql_thread_end();
 
@@ -1976,7 +1979,9 @@ end:
   pthread_cond_signal(&count_threshhold);
   pthread_mutex_unlock(&counter_mutex);
 
-  DBUG_RETURN(0);
+  DBUG_LEAVE;
+  my_thread_end();
+  return 0;
 }
 
 int
@@ -2008,6 +2013,7 @@ parse_option(const char *origin, option_string **stmt, char delm)
 
     count++;
     strncpy(buffer, ptr, (size_t)(retstr - ptr));
+    buffer[retstr - ptr]= 0;
     if ((buffer_ptr= strchr(buffer, ':')))
     {
       char *option_ptr;
